/*
* FdfTk.h
*
*ADOBE SYSTEMS INCORPORATED
*Copyright [1999] - [2001] Adobe Systems Incorporated
*All Rights Reserved

*NOTICE:  Adobe permits you to use, modify, and distribute 
*this file in accordance with the terms of the Adobe license 
*agreement accompanying it.  If you have received this file 
*from a source other than Adobe, then your use, modification, 
*or distribution of it requires the prior written permission of Adobe.
*/

#ifndef _H_FDFTK
#define _H_FDFTK


#ifdef __cplusplus
extern "C" {
#endif


#ifndef TOOLKIT

#ifdef WIN32
#define FDFLIBAPI __declspec(dllimport)
#else
#define FDFLIBAPI
#endif

/*
* Basic types
*/

/* integer types */
typedef long int			ASInt32;

/* cardinal types */
typedef unsigned char       ASUns8;
typedef unsigned long int	ASUns32;

/* boolean */
typedef unsigned short int	ASBool;

#ifndef true
#define true				1
#endif
#ifndef false
#define false				0
#endif

typedef void* FDFDoc;

#else /* TOOLKIT */

typedef CosDoc FDFDoc;

#ifdef WIN32
#define FDFLIBAPI __declspec(dllexport)
#else
#define FDFLIBAPI
#endif

#endif /* TOOLKIT */


/* IMPORTANT NOTE about all strings in this API: All strings must be null-terminated.
   Every call specifies which encodings are acceptable for every string. Strings that are in Host-
   or PDFDocEncoding end with a single null byte. Strings in Unicode have '\xfe' as byte 0
   and '\xff' as byte 1. Additionally, they must be terminated with 2 null bytes.
   Field names are always in PDFDoc encoding, and they always represent the fully qualified
   name of the field (e.g. "employee.name.last").
   The PDFDocEncoding is described in Appendix D of the PDF Reference, Second Edition.
*/ 


typedef enum {
	FDFErcOK = 0,
	FDFErcInternalError,/* 1 - Internal FDF Library error */
	FDFErcBadParameter,	/* 2 - One or more of the parameters passed to the
						   function are invalid */
	FDFErcFileSysErr,	/* 3 - Some error using the file system,including
						   "file not found" */
	FDFErcBadFDF,		/* 4 - The FDF file being opened/parsed is invalid */
	FDFErcFieldNotFound,/* 5 - The field whose name was passed in parameter
						   "fieldName" does not exist in the FDF */
	FDFErcNoValue,		/* 6 - The field whose value was requested has no value */
	FDFErcEnumStopped,	/* 7 - Enumeration was stopped by FDFEnumValuesProc by
						   returning false */
	FDFErcCantInsertField, /* 8 - The field whose name was passed in parameter
						   "fieldName" can't be inserted into the FDF. This
						   might happen, for example, if you try to insert
						   "a.b" into an FDF that already has a field such as
						   "a.b.c" in it. Or, conversely, if you try to insert
						   "a.b.c" into an FDF that already has "a.b" */
	FDFErcNoOption,		/* 9 - The requested element in a field's /Opt key does
						   not exist, or the field has no /Opt key */
	FDFErcNoFlags,		/* 10 - The field has no /F or /Ff keys */
	FDFErcBadPDF,		/* 11 - The PDF passed as parameter to FDFSetAP is
						   invalid, or doesn't contain pageNum */
	FDFErcBufTooShort,	/* 12 - The buffer passed as parameter is too short
						   for the length of the data that the function wants
						   to return in it */
	FDFErcNoAP,			/* 13 - The field has no /AP key */
	FDFErcIncompatibleFDF, /* 14 - Cannot mix "classic" and "template-based" FDF
						   (see FDFAddTemplate) */
	FDFErcNoAppendSaves, /* 15 - The FDF does not include a /Differences key */
	FDFErcValueIsArray, /* 16 - The value of this field is an array. Use
						   FDFGetNthValue */
	FDFErcEmbeddedFDFs, /* 17 - The FDF that you passed as parameter is a container
						   for one or more FDFs embedded within it. Use
						   FDFOpenFromEmbedded to gain access to each embedded FDF */
	FDFErcNoMoreFDFs,	/* 18 - Returned by FDFOpenFromEmbedded when parameter iWhich
						   >= to the number of embedded FDFs (including the case when
						   the passed FDF does not contain any embedded FDFs) */
	FDFErcInvalidPassword,/* 19 - Returned by FDFOpenFromEmbedded when the embedded FDF
						   is encrypted, and the correct password to open it was not
						   provided */
	FDFErcLast
} FDFErc;

typedef enum {
	FDFValue = 0,	/* FDFGetValue, FDFSetValue */
	FDFStatus,		/* FDFGetStatus, FDFSetStatus */
	FDFFile,		/* FDFGetFile, FDFSetFile */
	FDFID,			/* FDFGetID, FDFSetID */
	FDFOpt,			/* FDFGetOpt, FDFSetOpt */
	FDFFf,			/* FDFGetFlags, FDFSetFlags */
	FDFSetFf,		/* FDFGetFlags, FDFSetFlags */
	FDFClearFf,		/* FDFGetFlags, FDFSetFlags */
	FDFFlags,		/* FDFGetFlags, FDFSetFlags */
	FDFSetF,		/* FDFGetFlags, FDFSetFlags */
	FDFClrF,		/* FDFGetFlags, FDFSetFlags */
	FDFAP,			/* FDFGetAP, FDFSetAP */
	FDFAS,			/* FDFSetAS */
	FDFAction,		/* FDFSet<action name>Action */
	FDFAA,			/* FDFSet<action name>Action */
	FDFAPRef,		/* FDFSetAPRef */
	FDFIF,			/* FDFSetIF */
	FDFTargetFrame,	/* FDFSetTargetFrame */
	FDFEncoding,	/* FDFGetEncoding, FDFSetEncoding */
	FDFJavaScript,	/* FDFAddDocJavaScript, FDFSetOnImportJavaScript */
	FDFAppendSaves	/* FDFExtractAppendSaves */
} FDFItem;

typedef enum {
	FDFNormalAP = 0,
	FDFRolloverAP,
	FDFDownAP
} FDFAppFace;

typedef enum {
	FDFAlways = 0,
	FDFTooSmall,
	FDFTooBig,
	FDFNever
} FDFScaleWhen;

typedef enum {
	FDFEnter = 0,
	FDFExit,
	FDFDown,
	FDFUp,
	FDFFormat,
	FDFValidate,
	FDFKeystroke,
	FDFCalculate,
	FDFOnFocus,
	FDFOnBlur
} FDFActionTrigger;


typedef struct {
    /* These are all the possible components of a pdf file spec. See the PDF Reference,
	   Second Edition, section "3.10 File Specifications" */
    char* FS;
    char* F;
    char* Mac;
    char* DOS;
    char* Unix;
    char* ID[2];
    ASBool bVolatile;
} pdfFileSpecRec, *pdfFileSpec;

#define SIMPLE_FILESPEC(fileSpec, fileName) \
    fileSpec.FS = NULL; \
    fileSpec.F = fileName; \
    fileSpec.Mac = NULL; \
    fileSpec.DOS = NULL; \
    fileSpec.Unix = NULL; \
    fileSpec.ID[0] = NULL; \
    fileSpec.ID[1] = NULL; \
    fileSpec.bVolatile = false;


#ifdef UNIX_ENV

/*
  FDFRegisterThreadsafeCallback: registers the callbacks to threadsafe
  operations for lock, unlock, and destroy.  The clientData will be passed
  to each of these operations; typically the clientData is initialized (by the
  user, before calling FDFRegisterThreadsafeCallbacks) by a call to a mutex
  initialize operation (e.g. mutex_init).  Returns FDFErcOK.
*/
typedef void (*ThreadsafeCallback)( void* threadsafeObj );

FDFLIBAPI FDFErc FDFRegisterThreadsafeCallbacks( 
   ThreadsafeCallback lockProc,
   ThreadsafeCallback unlockProc,
   ThreadsafeCallback destroyProc,
   void *clientData );

#endif /* UNIX_ENV */


#ifndef WIN32

/*
  FDFInitialize: Initializes the FDF Library.  Client should call
  FDFInitialize() before using any other FDF Library calls. It should only be
  called once, by the first thread in the application.
  Returns FDFErcOK.
*/
FDFLIBAPI FDFErc FDFInitialize(void);


/*
  FDFFinalize: Finalizes the FDF Library.  Client should call
  FDFFinalize() after done with all other FDF Library calls. It should only be
  called once, by the last thread in the application.
  Returns FDFErcOK.
*/
FDFLIBAPI FDFErc FDFFinalize(void);

#endif /* not WIN32 */


/*
  FDFGetVersion: Returns a constant string with the current version of the
  FdfTk (e.g. "5.0").
*/
FDFLIBAPI const char* FDFGetVersion(void);


/*
  FDFGetFDFVersion: Returns a constant string with the version of the FDF. As of
  Acrobat 5, possible return values are: "1.2", "1.3", "1.4". These numbers
  correspond to the version of the "PDF Reference" where the various FDF features
  were first introduced. The return value indicates which version describes all the
  features actually used in the FDF passed as parameter.
*/
FDFLIBAPI const char* FDFGetFDFVersion(FDFDoc theFDF);


/* 
  FDFSetFDFVersion: You can call this method with either "1.2", "1.3", "1.4". However,
  it is particulary important to do so when the FDF being generated utilizes features
  first introduced with the two latter versions.
  Parameters:

  - theFDF
  - cVersion: One of "1.2", "1.3", "1.4"
  - Error codes: FDFErcBadParameter, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetFDFVersion(FDFDoc theFDF, const char* cVersion);



/*
  FDFCreate: Creates a new FDF. Client should call FDFClose() when the FDF is
  no longer needed. Parameters:

  - pTheFDF: If FDFCreate() returns FDFErcOK, then pTheFDF will point to an
    FDFDoc, which is needed for most other calls in the API.
  - Error codes: FDFErcBadParameter, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFCreate(FDFDoc* pTheFDF);


/*
  FDFOpen: Reads an FDF file into memory. Client should call FDFClose() when
  the FDF is no longer needed. Parameters:

  - fileName: Complete pathname (in Host encoding), or	"-" to read from stdin.
  - howMany: If fileName specifies stdin, then howMany should indicate the
 	number of characters to read. Otherwise, it is unused. In a web server
 	environment, this is available as the value of the CONTENT_LENGTH
 	environment variable. In some servers executing cgi-bin scripts, if the
 	script tries to read stdin until an EOF is reached, the script hangs.
 	Thus this parameter.
  - pTheFDF: If FDFOpen() returns FDFErcOK, then pTheFDF will point to an
    FDFDoc, which is needed for most other calls in the API.
  - Error codes: FDFErcBadParameter, FDFErcFileSysErr, FDFErcBadFDF,
	FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFOpen(const char* fileName, ASInt32 howMany, FDFDoc* pTheFDF);


/*
  FDFOpenFromEmbedded: Use this call when you discover that the FDF you are dealing with
  is only a container for one or more "real" FDFs. You discover this when any of the
  calls return FDFErcEmbeddedFDFs. Client should call FDFClose() when the FDF returned
  by this call is no longer needed (in addition to calling FDFClose() on the container).
  Parameters:

  - theFDF
  - iWhich: Zero-based index of the embedded FDF that should be opened
  - cPassword: If the embedded FDF is encrypted, you need to pass the correct
    password in order to open it. If it's not encrypted, this parameter is ignored.
	The password consists of an array of bytes, and is terminated by a single NULL.
  - pTheEmbeddedFDF: If FDFOpenFromEmbedded() returns FDFErcOK, then pTheEmbeddedFDF will
    point to the opened FDFDoc, which can then be used for most other calls in the API.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcIncompatibleFDF, FDFErcNoMoreFDFs,
    FDFErcInvalidPassword, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFOpenFromEmbedded(FDFDoc theContainerFDF, ASInt32 iWhich,
									 const char* cPassword, FDFDoc* pTheEmbeddedFDF);


/*
  FDFClose: Frees resources used by the FDF. Before exiting, client should
  call FDFClose() for each open FDF. Client should not use the FDFDoc
  after the FDF has been closed. Parameters:

  - theFDF: Valid FDFDoc returned from FDFCreate, FDFOpen, or FDFOpenFromEmbedded
  - Error codes: FDFErcBadParameter
*/
FDFLIBAPI FDFErc FDFClose(FDFDoc theFDF);


/*
  FDFEmbedAndClose: Embeds an FDF inside another, and then closes it. A container FDF can
  be a carrier for multiple embedded FDFs within it. The container FDF cannot not be a
  templates-based FDF (if it is, FDFErcIncompatibleFDF is returned). The FDF to embed cannot
  itself be a container FDF (if it is, FDFErcEmbeddedFDFs is returned), or be a
  templates-based FDF (if it is, FDFErcIncompatibleFDF is returned).
  Each of the embedded FDFs may optionally be password protected.
  A container FDF can optionally include, besides the embedded FDFs, only an /F and/or an
  /ID keys (see FDFSetFile and FDFSetID). Any other desired attributes belong in the
  embedded FDFs themselves. In turn, any /F or /ID keys in the embedded FDFs are ignored
  by Acrobat on FDF import, since it only looks for those keys in the container FDF.
  This call can be used multiple times for the same container FDF. Each successive FDF is
  embedded after any previous FDFs already present in the container.
  Parameters:

  - theFDF
  - theEmbeddedFDF: Gets automatically closed by this call, unless FDFErcBadParameter
    gets returned.
  - cPassword: If the embedded FDF should be stored encrypted, you need to provide the
    password. The password consists of an array of bytes, and is terminated by a single NULL.
	If you pass NULL, or an array whose first byte is NULL, the FDF will be embedded unencrypted..
  - Error codes: FDFErcBadParameter, FDFErcIncompatibleFDF, FDFErcEmbeddedFDFs,
    FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFEmbedAndClose(FDFDoc theContainerFDF, FDFDoc theEmbeddedFDF,
								  const char* cPassword);


/*
  FDFSave: Writes out an FDF file. Parameters:

  - theFDF: Valid FDFDoc returned from FDFCreate, FDFOpen, or FDFOpenFromEmbedded
  - fileName: Complete pathname (in Host encoding), or	"-" to write to stdout.
    The FDF cannot be saved to a file that was opened with FDFOpen() and hasn't
	yet been closed with FDFClose().
  - Error codes: FDFErcBadParameter, FDFErcFileSysErr, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSave(FDFDoc theFDF, const char* fileName);


/*
  FDFNextFieldName: Used to enumerate the field names in the FDF, this function
  returns the one that comes after the one passed as parameter (or the first
  one, if none is passed). The client can then use the returned field name as
  parameter to many of the calls in this API. See FDFEnumValues() for an
  alternative way to enumerate all the fields in the FDF. Parameters:

  - theFDF
  - fieldName: Name of the previous field, such that
	FDFNextFieldName() will return (in nextFldName) the one that comes after
	it in the FDF. If fieldName is NULL, the first field name in the FDF is
	returned.
  - nextFldName: Filled by FDFNextFieldName(), if it returns FDFErcOK. Buffer
	containing the next field name in the FDF. If we have reached the end of
	the FDF, nextFldName will contain the empty string (and nBytes will be 0).
	If nextFldName is NULL, the	function will just return in nBytes the number
	of bytes required.
	It is OK to pass the same buffer for the fieldName and nextFldName
	parameters.
  - szNextFldName: The maximum number of bytes that can be written into
	nextFldName. If	nextFldName is NULL, then szNextFldName is unused.
  - nBytes:	FDFNextFieldName() sets this value to zero before doing any work or
	error checking.
	If nextFldName is NULL and FDFNextFieldName() returns FDFErcOK,
	or nextFldName is not NULL and FDFNextFieldName() returns
	FDFErcBufTooShort, the number of bytes in the next field's name is
	returned in nBytes.
	Otherwise, if FDFNextFieldName() returns FDFErcOK and nextFldName is not
	NULL, the number of bytes copied into it is	returned, excluding the
	terminating '\0'. You must pass at least the length	+ 1 as the buffer
	size since the routine adds a '\0' terminator.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcIncompatibleFDF,
	FDFErcEmbeddedFDFs, FDFErcFieldNotFound, FDFErcBufTooShort, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFNextFieldName(FDFDoc theFDF, const char* fieldName,
								  char* nextFldName,
								  ASInt32 szNextFldName,
								  ASInt32* nBytes);

/*
  Callback for FDFEnumValues. Called once for each field value. Parameters:

  - bufFldName: Buffer containing a field name. This buffer
  	was supplied in the call to FDFEnumValues, and is filled by that function
  	with a new field name for each invocation of the callback.
  - bufFldVal: Buffer containing a field value. This buffer
 	was supplied in the call to FDFEnumValues, and is filled by that function
 	with a new value for each invocation of the callback. If the FDF includes
	an /Encoding key (see FDFGetEncoding), then the returned value will be in
	that encoding. Otherwise, the value will be in either PDFDocEncoding, or in
	Unicode if it cannot be represented in PDFDocEncoding (i.e. it contains
	double-byte characters).
  - clientData: User-supplied data that was passed in the call to
	FDFEnumValues.
  - Return value: true to continue enumeration, false to halt it.
*/
typedef ASBool (*FDFEnumValuesProc)(char* bufFldName, char* bufFldVal,
									void* clientData);


/*
  FDFEnumValues: Enumerates the field values in the FDF by calling a
  user-supplied procedure for each one.  See FDFNextFieldName() for an
  alternative way to enumerate all the fields in the FDF. Parameters:

  - theFDF
  - enumValuesProc: User-supplied callback that is called for each field
	value. Enumeration halts if enumValuesProc returns false.
  - bufFldName: Buffer that will contain each field name as enumValuesProc
	gets called.
  - szBufFldName: The maximum number of bytes that can be written into
	bufFldName.
  - bufFldVal: Buffer that will contain each field value as enumValuesProc
	gets called.
  - szBufFldVal: The maximum number of bytes that can be written into
	bufFldVal.
  - clientData: Pointer to user-supplied data that is passed to enumValuesProc
	each time it is called.
  - bSkipEmpty: If true, will skip during the enumeration fields that don't
	have a value, or whose value is an array. Otherwise, for such fields will call
	enumValuesProc with bufFldVal containing the empty string.
  - Error codes: FDFErcBadParameter, FDFErcEnumStopped, FDFErcBadFDF,
	FDFErcIncompatibleFDF, FDFErcEmbeddedFDFs, FDFErcBufTooShort,
	FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFEnumValues(FDFDoc theFDF, FDFEnumValuesProc enumValuesProc,
							   char* bufFldName, ASInt32 szBufFldName,
							   char* bufFldVal, ASInt32 szBufFldVal,
							   void* clientData, ASBool bSkipEmpty);


/* 
  FDFGetValue: Gets the value of a field (i.e. the value of /V) as a string.
  Parameters:

  - theFDF
  - fieldName
  - buffer: Filled by FDFGetValue(), if it returns FDFErcOK. Buffer
	containing the value of the field. If buffer is NULL, the function will
	just return in nBytes the number of	bytes required. If the FDF includes
	an /Encoding key (see FDFGetEncoding), then the returned value will be in
	that encoding. Otherwise, the value will be in either PDFDocEncoding, or in
	Unicode if it cannot be represented in PDFDocEncoding (i.e. it contains
	double-byte characters).
  - bufSize: The maximum number of bytes that can be written into buffer. If
	buffer is NULL, then bufSize is unused.
  - nBytes:	FDFGetValue() sets this value to zero before doing any work or
	error checking.
	If buffer is NULL and FDFGetValue() returns FDFErcOK, or buffer
	is not NULL and FDFGetValue() returns FDFErcBufTooShort, the number of
	bytes in the specified field's value is	returned in nBytes.
	Otherwise, if FDFGetValue() returns FDFErcOK and buffer is not NULL, the
	number of bytes copied into it is returned, excluding the null terminator.
	You must pass at least the length + 2 as the buffer	size since the routine
	adds a null terminator (which consists of 2 null bytes in the case of Unicode).
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcIncompatibleFDF,
	FDFErcEmbeddedFDFs, FDFErcFieldNotFound, FDFErcNoValue, FDFErcValueIsArray,
	FDFErcBufTooShort, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFGetValue(FDFDoc theFDF, const char* fieldName,
							 char* buffer, ASInt32 bufSize, ASInt32* nBytes);


/* 
  FDFGetNthValue: Use this call when FDFGetValue returns FDFErcValueIsArray.
  Parameters:

  - iWhich: Index (zero-based) of the desired value. This call can be used instead of
    FDFGetValue, even if the field's value is not an array, in which case iWhich is ignored.
	If iWhich is greater than the last element in the array, this call returns FDFErcNoValue.
  - Rest of the parameters: Same as FDFGetValue.
*/
FDFLIBAPI FDFErc FDFGetNthValue(FDFDoc theFDF, const char* fieldName, ASInt32 iWhich,
								char* buffer, ASInt32 bufSize, ASInt32* nBytes);


/* 
  FDFSetValue: Sets the value of a field (i.e. the value of /V). If the field
  does not yet exist in the FDF, it is created. If it does, and already has a
  value, that old value is replaced. If parameter "theFDF" is a "template-based"
  FDF (i.e. an FDF for which FDFAddTemplate has been called), then FDFSetValue
  acts on the most recently added template. Parameters:

  - theFDF
  - fieldName
  - newValue: If the FDF includes an /Encoding key (see FDFSetEncoding), then the
    string you pass should be in that encoding. Otherwise, the value should be in
	either PDFDocEncoding, or in Unicode if it cannot be represented in PDFDocEncoding
	(i.e. it contains double-byte characters).
	Hint: For radio buttons and checkboxes, newValue must be either "Off",
	or a value that was entered as the "Export Value" when defining the
	properties of the field.
  - bNotUsed: This parameter is not used by this method. You can pass true or false.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetValue(FDFDoc theFDF, const char* fieldName,
							 const char* newValue, ASBool bNotUsed);


/* 
  FDFSetValues: Sets the value of a field to an array. This only works for fields
  of type "listbox" that have the "Multiple Selection" option enabled.
  Parameters:

  - theFDF
  - nValues: An integer specifying how many values are being passed in
	parameter "newValues". If only one, this call is identical to FDFSetValue.
  - newValues: An array of strings.
  - Rest of the parameters: Same as FDFSetValue.
*/
FDFLIBAPI FDFErc FDFSetValues(FDFDoc theFDF, const char* fieldName, ASInt32 nValues,
							  const char* newValues[]);


/* 
  FDFGetStatus: Gets the status of the FDF (i.e. the value of /Status).
  Note: FDF exported from an Acrobat Form does not contain a /Status. However,
  FDFGetStatus() can still be useful when parsing an FDF produced some other
  way. For example, the FDF may have previously been produced with this very
  Library, and saved to a file. Parameters:

  - theFDF
  - buffer: Filled by FDFGetStatus(), if it returns FDFErcOK. Buffer containing
	the value of /Status. If the FDF contains no /Status, buffer will contain
	the empty string. If buffer is NULL, the function will just return in
	nBytes the number of bytes required. The returned string is Host-encoded.
  - bufSize: The maximum number of bytes that can be written into buffer. If
	buffer is NULL, then bufSize is unused.
  - nBytes:	FDFGetStatus() sets this value to zero before doing any work or
	error checking.
	If buffer is NULL and FDFGetStatus() returns FDFErcOK, or buffer
	is not NULL and FDFGetStatus() returns FDFErcBufTooShort, the number of
	bytes in the value of /Status is returned in nBytes.
	Otherwise, if FDFGetStatus() returns FDFErcOK and buffer is not NULL, the
	number of bytes copied into it is returned, excluding the terminating '\0'.
	You must pass at least the length + 1 as the buffer	size since the routine
	adds a '\0' terminator.
  - Error codes: FDFErcBadParameter, FDFErcIncompatibleFDF, FDFErcEmbeddedFDFs,
    FDFErcBufTooShort, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFGetStatus(FDFDoc theFDF, char* buffer, ASInt32 bufSize,
							  ASInt32* nBytes);


/* 
  FDFSetStatus: Sets the status of the FDF (i.e. the value of /Status). If the
  FDF already has a /Status key, its old value is replaced. Parameters:

  - theFDF
  - newStatus: Host-encoded string to use as new status.
  - Error codes: FDFErcBadParameter, FDFErcIncompatibleFDF, FDFErcEmbeddedFDFs,
    FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetStatus(FDFDoc theFDF, const char* newStatus);


/* 
  FDFGetFile: Gets the value of the FDF's /F key. This value is assumed to be
  of type PDF String, since that is what Acrobat produces when exporting FDF. An /F
  key generated by FDFSetFileEx below cannot be read back with FDFGetFile, and
  would cause FDFGetFile to return nBytes set to zero.
  Parameters:

  - theFDF
  - buffer: Filled by FDFGetFile(), if it returns FDFErcOK. Buffer containing
	the value of the FDF's /F key. The encoding is Host.
 	If the FDF contains no /F, or the value of /F is not a PDF String, then
 	buffer will contain the empty string. If buffer is NULL, the function will
 	just return in nBytes the number of bytes required.
  - bufSize: The maximum number of bytes that can be written into buffer. If
	buffer is NULL, then bufSize is unused.
  - nBytes:	FDFGetFile() sets this value to zero before doing any work or
	error checking.
	If buffer is NULL and FDFGetFile() returns FDFErcOK, or buffer
	is not NULL and FDFGetFile() returns FDFErcBufTooShort, the number of
	bytes in the value of the FDF's /F key is returned in nBytes.
	Otherwise, if FDFGetFile() returns FDFErcOK and buffer is not NULL, the
	number of bytes copied into it is returned, excluding the terminating '\0'.
	You must pass at least the length + 1 as the buffer	size since the routine
	adds a '\0' terminator.
  - Error codes: FDFErcBadParameter, FDFErcIncompatibleFDF, FDFErcBufTooShort,
	FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFGetFile(FDFDoc theFDF, char* buffer, ASInt32 bufSize,
							ASInt32* nBytes);


/* 
  FDFSetFile: Sets the value of the FDF's /F key. If the FDF already has an /F
  key, its old value is replaced. Parameters:

  - theFDF
  - newFile: Host-encoded string to use as the new value of the /F key.
    Note that the value of /F can be a URL, and at FDF import time Acrobat will
    use WebLink to retrieve it. It can be a relative path (or URL) to the path
	of the FDF.
  - Error codes: FDFErcBadParameter, FDFErcIncompatibleFDF, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetFile(FDFDoc theFDF, const char* newFile);


/* 
  FDFSetTargetFrame: Sets the value of the FDF's /Target key. This key is only relevant
  if the FDF has an /F key as well (see FDFSetFile or FDFSetFileEx). The /Target
  key is useful to cause the PDF (the one indicated by the /F key) to open in a particular
  browser frame (this key is equivalent to the "target" attribute in HTML). Parameters:

  - theFDF
  - cTargetFrame: Value of the /Target key. Should be Host-encoded.
  - Error codes: FDFErcBadParameter, FDFErcIncompatibleFDF, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetTargetFrame(FDFDoc theFDF, const char* cTargetFrame);


/* 
  FDFSetFileEx: Same as FDFSetFile, except that you can specify more complex
  file specifications. Parameters:

  - theFDF
  - fileSpec: The value for the /F key. It is only necessary to fill-in the
	fields of the pdfFileSpec data structure that you care about with
	null-terminated strings, and set the others to NULL. The most common case
	is to only use the F field, and to this end a convenience macro
	SIMPLE_FILESPEC is available (albeit in this case you would be better off
	simply using FDFSetFile).
  - Error codes: FDFErcBadParameter, FDFErcIncompatibleFDF, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetFileEx(FDFDoc theFDF, const pdfFileSpec fileSpec);


/* 
  FDFGetID: Gets the value of one element in the FDF's /ID key. Parameters:

  - theFDF
  - nElemNum: The element number to get from the FDFs /ID key. Must be one
	of the following: 0: the permanent ID; 1: the changing ID.
  - buffer: Filled by FDFGetID(), if it returns FDFErcOK. Buffer containing
	the requested ID element. If the FDF contains no /ID, then buffer will
	contain the empty string. If buffer is NULL, the function will just return
	in nBytes the number of bytes required.
  - bufSize: The maximum number of bytes that can be written into buffer. If
	buffer is NULL, then bufSize is unused.
  - nBytes:	FDFGetID() sets this value to zero before doing any work or
	error checking.
	If buffer is NULL and FDFGetID() returns FDFErcOK, or buffer
	is not NULL and FDFGetID() returns FDFErcBufTooShort, the number of
	bytes of each ID element is returned in nBytes.
	Otherwise, if FDFGetID() returns FDFErcOK and buffer is not NULL, the
	number of bytes copied into it is returned.
	Note: this is *not* a null-terminated string.
  - Error codes: FDFErcBadParameter, FDFErcIncompatibleFDF, FDFErcBufTooShort,
	FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFGetID(FDFDoc theFDF, ASInt32 nElemNum, ASUns8* buffer,
						  ASInt32 bufSize, ASInt32* nBytes);


/* 
  FDFSetID: Sets the value of one element in the FDF's /ID key. If the FDF
  already has an /ID key, the requested element is replaced. Parameters:

  - theFDF
  - nElemNum: The element number to set in the FDFs /ID key. Must be one of
	the following: 0: the permanent ID; 1: the changing ID.
  - elem: Buffer containing the ID element to set.
  - bufSize: The number of bytes in elem.
  - Error codes: FDFErcBadParameter, FDFErcIncompatibleFDF, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetID(FDFDoc theFDF, ASInt32 nElemNum, const ASUns8* elem,
						  ASInt32 bufSize);


/* 
  FDFGetEncoding: Gets the value of the FDF's /Encoding key as a string. If this
  key exists, then all Values (see FDFEnumValues, FDFGetValue), as well as Options
  (see FDFGetOpt) within that FDF are encoded using the flavor of Host encoding
  indicated by that key. Possible encodings are: "Shift_JIS", "UHC", "GBK", "BigFive".
  If the FDF contains no /Encoding key, then Values and Options are all in either
  PDFDocEncoding, or in Unicode if they cannot be represented in PDFDocEncoding
  (i.e. they contain double-byte characters).
  Parameters:

  - theFDF
  - buffer: Filled by FDFGetEncoding(), if it returns FDFErcOK. Buffer containing
	the value of the FDF's /Encoding key as a string.
 	If the FDF contains no /Encoding, then buffer will contain the empty string.
	If buffer is NULL, the function will just return in nBytes the number of bytes
	required.
  - bufSize: The maximum number of bytes that can be written into buffer. If
	buffer is NULL, then bufSize is unused.
  - nBytes:	FDFGetEncoding() sets this value to zero before doing any work or
	error checking.
	If buffer is NULL and FDFGetEncoding() returns FDFErcOK, or buffer
	is not NULL and FDFGetEncoding() returns FDFErcBufTooShort, the number of
	bytes needed to represent as a string the value of the FDF's /Encoding key
	is returned in nBytes.
	Otherwise, if FDFGetEncoding() returns FDFErcOK and buffer is not NULL, the
	number of bytes copied into it is returned, excluding the terminating '\0'.
	You must pass at least the length + 1 as the buffer	size since the routine
	adds a '\0' terminator.
  - Error codes: FDFErcBadParameter, FDFErcIncompatibleFDF, FDFErcEmbeddedFDFs.
    FDFErcBufTooShort, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFGetEncoding(FDFDoc theFDF, char* buffer, ASInt32 bufSize,
								ASInt32* nBytes);


/* 
  FDFSetEncoding: Sets the value of the FDF's /Encoding key. If you are setting
  this key in the FDF, it is assumed that all Values (see FDFSetValue) and Options
  (see FDFSetOpt) in the FDF will follow that encoding.
  Parameters:

  - theFDF
  - newEncoding: Null-terminated string that will be converted to a PDF Name
	before making it the value of the /Encoding key. Acceptable encodings are:
	"Shift_JIS", "UHC", "GBK", "BigFive" (exact spelling required).
  - Error codes: FDFErcBadParameter, FDFErcEmbeddedFDFs, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetEncoding(FDFDoc theFDF, const char* newEncoding);


/* 
  FDFGetOpt: Gets the value of one element in a field's /Opt key.
  Note: FDF exported from an Acrobat Form does not contain an /Opt. However,
  FDFGetOpt() can still be useful when parsing an FDF produced some other
  way. For example, the FDF may have previously been produced with this very
  Library, and saved to a file. Parameters:

  - theFDF
  - fieldName
  - nElemNum: The element number to get from the fields /Opt array. The first
  	element in the /Opt array has an index of zero. If this parameter is -1,
  	then this function returns in nRet the number of elements in the field's
  	/Opt array, and buffer1, buffer2, bufSize are unused.
  - buffer1: Filled by FDFGetOpt(), if it returns FDFErcOK. If the /Opt array
	element requested is itself composed of an array containing two strings,
	then buffer1 contains the first one (which is the export
	value). If, on the other hand, the /Opt array element requested is a
	single string, then buffer1 contains it (which is the item name).
    If buffer1 is NULL, the function will just return in nRet the number of
    bytes required.
	See FDFGetEncoding for a discussion of encodings.
  - buffer2: Filled by FDFGetOpt(), if it returns FDFErcOK. If the /Opt array
	element requested is itself composed of an array containing two strings,
	then buffer2 contains the second one (which is the item
	name). Otherwise, buffer2 contains the empty string. If buffer1 is NULL,
	then buffer2 is unused.
  - bufSize: The maximum number of bytes that can be written into buffer1 and
	buffer2 (which are assumed to be the same size). If buffer1 is NULL, then
	bufSize is unused.
  - nRet: FDFGetOpt() sets this value to zero before doing any work or
	error checking.
	If nElemNum is -1 (and FDFGetOpt() returns FDFErcOK), the number of
	elements in the field's /Opt array is returned here. Else, if buffer1 is
	NULL and FDFGetOpt() returns FDFErcOK, or buffer1 is not NULL and
	FDFGetOpt() returns FDFErcBufTooShort, the number of bytes that each of
	buffer1 and buffer2 need to be is returned in nRet.
	Finally, if FDFGetOpt() returns FDFErcOK and buffer1 is not NULL, returns
	the number of bytes copied into	buffer1, excluding the null terminator.
	You must pass at least the length + 2 as the buffer size since the routine
	adds a null terminator (which consists of 2 null bytes in the case of Unicode).
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcIncompatibleFDF,
	FDFErcEmbeddedFDFs, FDFErcFieldNotFound, FDFErcNoOption, FDFErcBufTooShort,
	FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFGetOpt(FDFDoc theFDF, const char* fieldName,
						   ASInt32 nElemNum, char* buffer1, char* buffer2,
						   ASInt32 bufSize, ASInt32* nRet);


/* 
  FDFSetOpt: Sets the value of one element in a field's /Opt key. If the field
  does not yet exist in the FDF, it is created. If it does, and already has an
  /Opt key, the requested element in the array is replaced. If parameter
  "theFDF" is a "template-based" FDF (i.e. an FDF for which FDFAddTemplate has
  been called), then FDFSetOpt acts on the most recently added template.
  Note that an FDF used to dynamically  change the /Opt of a field in an Acrobat
  Form typically needs to also change the /V of that same field (i.e. use
  FDFSetValue when creating the FDF), particularly if the old /V of the
  field is not a member of the new /Opt being imported into the Form. Remember
  that if the /Opt includes item names and export values, then /V should use
  one of the export values, and not one of the item names.
  Parameters:

  - theFDF
  - fieldName
  - nElemNum: The index of the element to set in the fields /Opt array. The
	first element in the /Opt array has an index of zero.
  - string1: If the /Opt array element to be set will be composed of an array
	containing two strings, then string1 contains the first one
	(which is the export value). If, on the other hand, the /Opt array
	element will be a single string, then string1 contains it (which
	is the item name). See FDFSetEncoding for a discussion of encodings.
  - string2: If the /Opt array element to be set will be composed of an array
	containing two strings,	then string2 contains the second (null-terminated)
	string (which is the item name). Otherwise, string2 should be NULL.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetOpt(FDFDoc theFDF, const char* fieldName,
						   ASInt32 nElemNum, const char* string1,
						   const char* string2);


/* 
  FDFGetFlags: Gets the flags of a field (i.e. the value of one of: /Ff, /F).
  Note: FDF exported from an Acrobat Form does not contain flags. However,
  FDFGetFlags() can still be useful when parsing an FDF produced some other
  way. For example, the FDF may have previously been produced with this very
  Library, and saved to a file. Parameters:

  - theFDF
  - fieldName
  - whichFlags: Must be one of: FDFFf or FDFFlags.
  - pTheFlags: If FDFOpen() returns FDFErcOK, then pTheFDF will point to the
	requested flags.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcIncompatibleFDF, 
	FDFErcEmbeddedFDFs, FDFErcFieldNotFound, FDFErcNoFlags, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFGetFlags(FDFDoc theFDF, const char* fieldName,
							 FDFItem whichFlags, ASUns32* pTheFlags);


/* 
  FDFSetFlags: Sets the flags of a field (i.e. the value of one of: /Ff, /F,
  /SetFf, /ClrFf, /SetF, /ClrF). If the field does not yet exist in the FDF,
  it is created. If it does, and already has flags, the old value is replaced.
  If parameter "theFDF" is a "template-based" FDF (i.e. an FDF for which
  FDFAddTemplate has been called), then FDFSetFlags acts on the most recently
  added template.
  For the meaning of annotation flags, see the Flags description in the table
  describing "Annotation attributes" in the "PDF 1.2 Reference Manual":
  <http://www.adobe.com/supportservice/devrelations/PDFS/TN/PDFSPEC.PDFParameters>
  For the meaning of field-specific flags, see the "Attributes common to all types
  of fields" table in the "PDF 1.2 Reference Manual".
  Parameters:

  - theFDF
  - fieldName
  - whichFlags: Must be one of: FDFFf, FDFSetFf, FDFClearFf, FDFFlags, FDFSetF,
	FDFClrF.
  - newFlags: New value for flags.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetFlags(FDFDoc theFDF, const char* fieldName,
							 FDFItem whichFlags, ASUns32 newFlags);


/*
  FDFGetAP: Gets the appearance of a field (i.e. the value of one of the faces
  of the /AP key), and creates a PDF document out of it. Parameters:

  - theFDF
  - fieldName
  - whichFace: Which face of /AP is requested.
  - fileName: Host-encoded string for the pathname of the PDF file to create.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcIncompatibleFDF,
	FDFErcEmbeddedFDFs, FDFErcFieldNotFound, FDFErcFileSysErr, FDFErcNoAP,
	FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFGetAP(FDFDoc theFDF, const char* fieldName,
						  FDFAppFace whichFace, const char* fileName);


/*
  FDFSetAP: Sets the appearance of a field (i.e. the value of one of the faces
  of the /AP key) from a PDF document. If the field does not yet exist in the
  FDF, it is created. If it does, the requested face in the /AP is replaced.
  If parameter "theFDF" is a "template-based" FDF (i.e. an FDF for which
  FDFAddTemplate has been called), then FDFSetAP acts on the most recently
  added template.
  Notes: Acrobat will only import an /AP from FDF into fields of type "Button".
  In addition, only the FDFNormalAP (see parameter "whichFace" below) will be
  imported (and FDFDownAP, FDFRolloverAP will be ignored) unless the button that
  the /AP is being imported into has a "Highlight" of type "Push". Finally,
  be aware that the new imported /AP will not show if the "Layout" for the
  button is of type "Text only".
  Tip: If, once the FDF containing the new /AP is imported into the Acrobat
  Form, the picture looks too small inside the button field, with too much white
  space around it, you may want to crop (using Acrobat Exchange) the PDF page
  used as the source of the /AP (i.e. the one identified by parameters "fileName"
  and "pageNum" below).
  Parameters:

  - theFDF
  - fieldName
  - whichFace: Which face is to be set.
  - subAP: String in PDFDocEncoding to indicate which sub-appearance is to be
	set. If the appearance to be set has no sub-appearances, then pass NULL.
	Note: The current version of Acrobat does not import sub-appearances,
	therefore you should pass NULL for this parameter.
  - fileName: Host-encoded string for the pathname of the PDF file.
  - pageNum: Page within PDF file to be used for the appearance (the first page
	is 1).
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcBadPDF,
	FDFErcEmbeddedFDFs, FDFErcFileSysErr, FDFErcCantInsertField,
	FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetAP(FDFDoc theFDF, const char* fieldName,
						  FDFAppFace whichFace, const char* subAP,
						  const char* fileName, ASInt32 pageNum);


/*
  FDFSetAPRef: Sets a reference to the PDF document to use for the appearance
  of a field (i.e. the value of one of the faces within the /APRef key). If the
  field does not yet exist in the FDF, it is created. If it does, the requested
  face in the /APRef is replaced. If parameter "theFDF" is a "template-based"
  FDF (i.e. an FDF for which FDFAddTemplate has been called), then FDFSetAPRef
  acts on the most recently added template.
  See additional notes for FDFSetAP.
  Note that /AP and /APRef are mutually exclusive. If both appear inside an FDF,
  /AP takes precedence and /APRef is ignored. Note that while /AP inside an FDF
  corresponds to the /AP for a button field in an AcroForm, /APRef is an FDF-only
  key, and is resolved to an /AP during FDF import (whereas /AP is resolved
  during FDF creation). In other words, /APRef never shows up inside PDF.
  The value of /F within /APRef can be a URL. In addition, if the FDF will in
  fact be imported into an existing PDF (as opposed to being used to construct
  a brand new document), then the value of /F can be a relative file specification.
  /APRef doesn't have to contain an /F if the appearance resides (presumably as
  invisible template) in the PDF that the FDF will be imported into.
  Parameters:

  - theFDF
  - fieldName
  - whichFace: Which face is to be set.
  - fileSpec: The value for the /F key, which is the file specification for the
	PDF that contains the page to be used for the appearance.
	Can be NULL (in which case when the FDF gets imported into an Acrobat Form,
	the template is expected to reside inside that PDF file).
	If it is not NULL, then it is only necessary to fill-in the fields of the
	pdfFileSpec data structure that you care about with null-terminated strings,
	and set the others to NULL. The most common case is to only use the F field,
	and to this end a convenience macro SIMPLE_FILESPEC is available.
  - templateName: Name of the page ("template") within the PDF file specified by
	fileSpec, to be used for the appearance. Should be in PDFDocEncoding or Unicode.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetAPRef(FDFDoc theFDF, const char* fieldName,
							 FDFAppFace whichFace, const pdfFileSpec fileSpec,
							 const char* templateName);


/*
  FDFSetIF: Sets the Icon Fit that should be used for the appearance of a field
  (i.e. the value of the /IF key). If the field does not yet exist in the FDF,
  it is created. If it does, and already has an /IF key, its old value is
  replaced. The /IF key determines the placement of the PDF icon used as the
  appearance of a button (see also FDFSetAP and FDFSetAPRef). If the FDF does
  not include an /IF, then the icon is placed according to the /IF already in
  the Form (or the default, if that field in the Form does not have an /IF).
  If parameter "theFDF" is a "template-based" FDF (i.e. an FDF for which
  FDFAddTemplate has been called), then FDFSetIF acts on the most recently
  added template.  Parameters:

  - theFDF
  - fieldName
  - scaleWhen: When to scale the icon to fit within the rectangle of the
	annotation that represents the field.
  - bProportional: How to do the scaling (if at all): proportionally or not.
  - x: A value between 0 and 1. 0 places the icon at the left edge, and 1 at
    the right edge, of the annotation rectangle.
  - y: A value between 0 and 1. 0 places the icon at the bottom edge, and 1 at
    the upper edge, of the annotation rectangle.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetIF(FDFDoc theFDF, const char* fieldName,
						  FDFScaleWhen scaleWhen, ASBool bProportional,
						  float x, float y);


/*
  FDFSetAS: Sets the value of the /AS key for a field. If the field does not
  yet exist in the FDF, it is created. If it does, and already has an /AS key,
  its old value is replaced.
  Note: do not use FDFSetAS for fields of type checkbox or radio button. Use
  FDFSetValue instead. When the value (i.e. /V) of a field of one of these two
  types changes (to a valid value, i.e. either "Off", or a value that was
  entered as the "Export Value" when defining the properties of the field), the
  /AS key automatically gets set to the correct value by Acrobat. You can use
  FDFSetAS for fields of type button, but only if they have subfaces defined.
  Currently Acrobat does not have a user interface that allows for defining
  subfaces for buttons. This could be accomplished using pdfmark instead (see
  <http://partners.adobe.com/asn/developer/acrosdk/docs/pdfmark.pdf>).
  Parameters:

  - theFDF
  - fieldName
  - newAS: String in PDFDocEncoding that will be converted to a PDF Name to use
	as the new value of /AS.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcIncompatibleFDF, FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetAS(FDFDoc theFDF, const char* fieldName,
						  const char* newAS);


/*
  FDFSetSubmitFormAction: Sets the value of either the /A or /AA keys of a
  field to an action of type SubmitForm, using the passed URL. The created
  action will not include a /Fields key. If the field does not yet exist in
  the FDF, it is created. If it does, and already has an /A (or /AA) key, its
  old value is replaced. If parameter "theFDF" is a "template-based" FDF (i.e.
  an FDF for which FDFAddTemplate has been called), then FDFSetSubmitFormAction
  acts on the most recently added template.  Parameters:

  - theFDF
  - fieldName
  - whichTrigger: If FDFUp, then an /A entry will be used, otherwise an /AA
	entry will be created.
	Note: Only the FDFEnter, FDFExit, FDFDown, FDFUp, FDFOnFocus and FDFOnBlur
	triggers can be used with a SubmitForm action.
  - theURL: Host-encoded string to be used as value for the /F key in the
	SubmitForm action.
  - theFlags: Value for the /Flags key in the SubmitForm action. If zero, since
	the	meaning of each individual flag was carefully defined to mean the same
	as if the /Flags key wasn't there at all, no /Flags key is inserted.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetSubmitFormAction(FDFDoc theFDF, const char* fieldName,
										FDFActionTrigger whichTrigger,
 										const char* theURL, ASUns32 theFlags);


/*
  FDFSetSubmitByNameAction: Sets the value of either the /A or /AA keys of a
  field to an action of type SubmitForm, using the passed URL and field names.
  This is similar to FDFSetSubmitFormAction (see above), except that you can
  specify which fields to submit. If parameter "theFDF" is a "template-based"
  FDF (i.e. an FDF for which FDFAddTemplate has been called), then
  FDFSetSubmitByNameAction acts on the most recently added template.
  Parameters:

  - theFDF
  - fieldName
  - whichTrigger: If FDFUp, then an /A entry will be used, otherwise an /AA
	entry will be created.
  	Note: In the current version of Acrobat, only the FDFEnter, FDFExit,
	FDFDown, and FDFUp triggers can be used with a SubmitForm action.
  - theURL: Host-encoded string to be used as value for the /F key in the
	SubmitForm action.
  - theFlags: Value for the /Flags key in the SubmitForm action. If zero, since
	the	meaning of each individual flag was carefully defined to mean the same
	as if the /Flags key wasn't there at all, no /Flags key is inserted.
	Please check "SubmitForm Action" in the "PDF 1.2 Reference Manual" for the
	meaning of each bit in /Flags, including how to interpret the "fields"
	parameter (below) as either fields to submit, or to exclude from submitting.
  - nFlds: An integer specifying how many field names are being passed in
	parameter "fields". If zero, then FDFSetSubmitByNameAction is identical to
	FDFSetSubmitFormAction.
  - fields: An array of "nFlds" pointers to strings representing
	the names of the fields to submit (they will be the value of
	the /Fields key in the SubmitForm action).
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetSubmitByNameAction(FDFDoc theFDF, const char* fieldName,
										  FDFActionTrigger whichTrigger,
 										  const char* theURL, ASUns32 theFlags,
										  ASInt32 nFlds, const char* fields[]);


/*
  FDFSetResetFormAction: Sets the value of either the /A or /AA keys of a
  field to an action of type ResetForm. The created action will not include
  /Fields or /Flags keys. If the field does not yet exist in the FDF, it is
  created. If it does, and already has an /A (or /AA) key, its old value is
  replaced. If parameter "theFDF" is a "template-based" FDF (i.e. an FDF for
  which FDFAddTemplate has been called), then FDFSetResetFormAction acts on
  the most recently added template.  Parameters:

  - theFDF
  - fieldName
  - whichTrigger: If FDFUp, then an /A entry will be used, otherwise an /AA
	entry will be created.
	Note: Only the FDFEnter, FDFExit, FDFDown, FDFUp, FDFOnFocus and FDFOnBlur
	triggers can be used with a ResetForm action.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetResetFormAction(FDFDoc theFDF, const char* fieldName,
									   FDFActionTrigger whichTrigger);


/*
  FDFSetResetByNameAction: Sets the value of either the /A or /AA keys of a
  field to an action of type ResetForm, using the passed field names.
  This is similar to FDFSetResetFormAction (see above), except that you can
  specify which fields to reset. If parameter "theFDF" is a "template-based"
  FDF (i.e. an FDF for which FDFAddTemplate has been called), then
  FDFSetResetByNameAction acts on the most recently added template.
  Parameters:

  - theFDF
  - fieldName
  - whichTrigger: If FDFUp, then an /A entry will be used, otherwise an /AA
	entry will be created.
	Note: Only the FDFEnter, FDFExit, FDFDown, FDFUp, FDFOnFocus and FDFOnBlur
	triggers can be used with a ResetForm action.
  - theFlags: Value for the /Flags key in the ResetForm action. If zero, since
	the	meaning of each individual flag was carefully defined to mean the same
	as if the /Flags key wasn't there at all, no /Flags key is inserted.
	Please check "ResetForm Action" in the "PDF 1.2 Reference Manual" for the
	meaning of each bit in /Flags, including how to interpret the "fields"
	parameter (below) as either fields to reset, or to exclude from resetting.
  - nFlds: An integer specifying how many field names are being passed in
	parameter "fields". If zero, then FDFSetResetByNameAction is identical to
	FDFSetResetFormAction.
  - fields: An array of "nFlds" pointers to names of the fields to reset
    (they will be the value of the /Fields key in the ResetForm action).
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetResetByNameAction(FDFDoc theFDF, const char* fieldName,
										 FDFActionTrigger whichTrigger,
										 ASUns32 theFlags,
										 ASInt32 nFlds, const char* fields[]);


/*
  FDFSetImportDataAction: Sets the value of either the /A or /AA keys of a
  field to an action of type ImportData. If the field does not yet exist in
  the FDF, it is created. If it does, and already has an /A (or /AA) key, its
  old value is replaced. If parameter "theFDF" is a "template-based" FDF (i.e.
  an FDF for which FDFAddTemplate has been called), then FDFSetImportDataAction
  acts on the most recently added template.  Parameters:

  - theFDF
  - fieldName
  - whichTrigger: If FDFUp, then an /A entry will be used, otherwise an /AA
	entry will be created.
	Note: Only the FDFEnter, FDFExit, FDFDown, FDFUp, FDFOnFocus and FDFOnBlur
	triggers can be used with an ImportData action.
  - theFile: Host-encoded string to be used as value for the /F key in the
	ImportData action.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetImportDataAction(FDFDoc theFDF, const char* fieldName,
										FDFActionTrigger whichTrigger,
										const char* theFile);


/*
  FDFSetJavaScriptAction: Sets the value of either the /A or /AA keys of a
  field to an action of type JavaScript. If the field does not yet exist in
  the FDF, it is created. If it does, and already has an /A (or /AA) key, its
  old value is replaced. If parameter "theFDF" is a "template-based" FDF (i.e.
  an FDF for which FDFAddTemplate has been called), then FDFSetJavaScriptAction
  acts on the most recently added template.  Parameters:

  - theFDF
  - fieldName
  - whichTrigger: If FDFUp, then an /A entry will be used, otherwise an /AA
	entry will be created.
  - theScript: Must be in either PDFDocEncoding or in Unicode. Hint: Use "\r"
    as line separator within the script. Use "\t" for tabs.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetJavaScriptAction(FDFDoc theFDF, const char* fieldName,
										FDFActionTrigger whichTrigger,
										const char* theScript);


/*
  FDFSetGoToAction: Sets the value of either the /A or /AA keys of a
  field to an action of type GoTo using a named destination. If the field
  does not yet exist in the FDF, it is created. If it does, and already has an
  /A (or /AA) key, its  old value is replaced. If parameter "theFDF" is a
  "template-based" FDF (i.e. an FDF for which FDFAddTemplate has been called),
  then FDFSetGoToAction acts on the most recently added template.  Parameters:

  - theFDF
  - fieldName
  - whichTrigger: If FDFUp, then an /A entry will be used, otherwise an /AA
	entry will be created.
	Note: Only the FDFEnter, FDFExit, FDFDown, FDFUp, FDFOnFocus and FDFOnBlur
	triggers can be used with a GoTo action.
  - theDest: String in PDFDocEncoding or Unicode to be used as value for the
    /D key in the GoTo action.
  - isName: If true, then theDest is converted to a PDF Name before making it
	the value of /D. If false, theDest is set to a PDF String.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetGoToAction(FDFDoc theFDF, const char* fieldName,
								  FDFActionTrigger whichTrigger,
								  const char* theDest, ASBool isName);


/*
  FDFSetGoToRAction: Sets the value of either the /A or /AA keys of a field
  to an action of type GoToR using a named destination. If the field
  does not yet exist in the FDF, it is created. If it does, and already has an
  /A (or /AA) key, its  old value is replaced. If parameter "theFDF" is a
  "template-based" FDF (i.e. an FDF for which FDFAddTemplate has been called),
  then FDFSetGoToRAction acts on the most recently added template.  Parameters:

  - theFDF
  - fieldName
  - whichTrigger: If FDFUp, then an /A entry will be used, otherwise an /AA
	entry will be created.
	Note: Only the FDFEnter, FDFExit, FDFDown, FDFUp, FDFOnFocus and FDFOnBlur
	triggers can be used with a GoToR action.
  - theDest: String in PDFDocEncoding or Unicode to be used as value for the
    /D key in the GoToR action.
  - isName: If true, then theDest is converted to a PDF Name before making it
	the value of /D. If false, theDest is set to a PDF String.
  - theFile: Host-encoded string to be used as value for the /F key in the
	GoToR action.
  - addNewWindow: If true, will add a /NewWindow key to the GoToR action, set
	to the value passed in newWindow. If false, will not add it, and newWindow
	is ignored.
  - newWindow: Value for the /NewWindow key in the GoToR action. Only used if
	addNewWindow is true.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetGoToRAction(FDFDoc theFDF, const char* fieldName,
								   FDFActionTrigger whichTrigger,
								   const char* theDest, ASBool isName,
								   const char* theFile, ASBool addNewWindow,
								   ASBool newWindow);


/*
  FDFSetURIAction: Sets the value of either the /A or /AA keys of a field to
  an action of type URI. If the field does not yet exist in the FDF, it is
  created. If it does, and already has an /A (or /AA) key, its  old value is
  replaced. If parameter "theFDF" is a "template-based" FDF (i.e. an FDF for
  which FDFAddTemplate has been called), then FDFSetURIAction acts on the most
  recently added template.  Parameters:

  - theFDF
  - fieldName
  - whichTrigger: If FDFUp, then an /A entry will be used, otherwise an /AA
	entry will be created.
	Note: Only the FDFEnter, FDFExit, FDFDown, FDFUp, FDFOnFocus and FDFOnBlur
	triggers can be used with a URI action.
  - theURI: Host-encoded string to be used as value for the /URI key in
	the URI action.
  - isMap: Value for the /IsMap key in the URI action. If false, no /IsMap
	key is added.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetURIAction(FDFDoc theFDF, const char* fieldName,
								 FDFActionTrigger whichTrigger,
 								 const char* theURI, ASBool isMap);


/*
  FDFSetNamedAction: Sets the value of either the /A or /AA keys of a field
  to a Named action. If the field does not yet exist in the FDF, it is
  created. If it does, and already has an /A (or /AA) key, its  old value is
  replaced. If parameter "theFDF" is a "template-based" FDF (i.e. an FDF for
  which FDFAddTemplate has been called), then FDFSetNamedAction acts on the
  most recently added template.  Parameters:

  - theFDF
  - fieldName
  - whichTrigger: If FDFUp, then an /A entry will be used, otherwise an /AA
	entry will be created.
	Note: Only the FDFEnter, FDFExit, FDFDown, FDFUp, FDFOnFocus and FDFOnBlur
	triggers can be used with a Named action.
  - theName: String in PDFDocEncoding that will be converted to a PDF Name to
	use as the value of /N in the Named action..
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetNamedAction(FDFDoc theFDF, const char* fieldName,
								   FDFActionTrigger whichTrigger,
 								   const char* theName);


/*
  FDFSetHideAction: Sets the value of either the /A or /AA keys of a field to
  an action of type Hide. If the field does not yet exist in the FDF, it is
  created. If it does, and already has an /A (or /AA) key, its  old value is
  replaced. If parameter "theFDF" is a "template-based" FDF (i.e. an FDF for
  which FDFAddTemplate has been called), then FDFSetHideAction acts on the
  most recently added template.  Parameters:

  - theFDF
  - fieldName
  - whichTrigger: If FDFUp, then an /A entry will be used, otherwise an /AA
	entry will be created.
	Note: Only the FDFEnter, FDFExit, FDFDown, FDFUp, FDFOnFocus and FDFOnBlur
	triggers can be used with a Hide action.
  - theTarget: String in PDFDocEncoding to be used as value for the /T key in
	the Hide action.
  - isHide: Value for the /H key in the Hide action. If true, no /H key is
	added.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetHideAction(FDFDoc theFDF, const char* fieldName,
								  FDFActionTrigger whichTrigger,
 								  const char* theTarget, ASBool isHide);


/*
  FDFSetStateAction: Sets the value of either the /A or /AA keys of a field to
  an action of type SetState. If the field does not yet exist in the FDF, it is
  created. If it does, and already has an /A (or /AA) key, its  old value is
  replaced. If parameter "theFDF" is a "template-based" FDF (i.e. an FDF for
  which FDFAddTemplate has been called), then FDFSetStateAction acts on the
  most recently added template.  Parameters:

  - theFDF
  - fieldName
  - whichTrigger: If FDFUp, then an /A entry will be used, otherwise an /AA
	entry will be created.
	Note: Only the FDFEnter, FDFExit, FDFDown, FDFUp, FDFOnFocus and FDFOnBlur
	triggers can be used with a SetState action.
  - theTarget: String in PDFDocEncoding to be used as value for the /T key in
	the SetState action.
  - theAS: String in PDFDocEncoding that will be converted to a PDF Name to use
	as the value of /AS in the SetState action.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcEmbeddedFDFs,
    FDFErcCantInsertField, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetStateAction(FDFDoc theFDF, const char* fieldName,
								   FDFActionTrigger whichTrigger,
 								   const char* theTarget, const char* theAS);


/*
  FDFRemoveItem: Removes an item (i.e. a key-value pair) from the FDF.
  If the item to be removed is not present, no error occurs (the function
  returns FDFErcOK). Parameters:

  - theFDF
  - fieldName: Name of the field from which the key-value pair is
	to be removed. If the item to be removed is not specific to one particular
	field (i.e. FDFStatus, FDFFile, FDFID), then pass NULL.
  - whichItem: Item to be removed.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcIncompatibleFDF,
	FDFErcEmbeddedFDFs, FDFErcFieldNotFound, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFRemoveItem(FDFDoc theFDF, const char* fieldName,
							   FDFItem whichItem);


/*
  FDFAddTemplate: Adds a template to the FDF. There are two types of FDF: the
  "classic" kind first introduced with Acrobat 3.0, and the "template-based"
  kind that constitutes the "recipe" for, upon import into Acrobat, directing
  the construction of a brand new PDF document from templates
  found inside specified PDF documents. You cannot add templates to a "classic"
  FDF: they are mutually exclusive (and you will get FDFErcIncompatibleFDF if
  you try). Many of the calls in the FDF Library (e.g. FDFSetFile) are
  incompatible with a "template-based" FDF (i.e. an FDF for which FDFAddTemplate
  has been called), and will as well return FDFErcIncompatibleFDF if called with
  such an FDF. Other calls (e.g. FDFSetValue) are ok with either kind of FDF.
  If called with a "template-based" FDF, they act on the most recently added
  template.  Parameters:

  - theFDF
  - bNewPage: If true, the template is used to start a new page, otherwise, it
    is appended to the last page (if any: if this is the very first template
	added to the FDF, this parameter is ignored). Note that when a "template-
	based" FDF is used to construct a brand new PDF document, each page can be
	the result of overlaying multiple templates one after another.
  - fileSpec: The value for the /F key within the /TRef key, which is the file
    specification for the PDF where the template is.
	Can be NULL (in which case when the FDF gets imported into Acrobat, the
	template is expected to reside inside the PDF file currently being viewed,
	which will get replaced as the top most document with the new PDF
	constructed as a result of importing the FDF).
	If it is not NULL, then it is only necessary to fill-in the fields of the
	pdfFileSpec data structure that you care about with null-terminated strings,
	and set the others to NULL. The most common case is to only use the F field,
	and to this end a convenience macro SIMPLE_FILESPEC is available.
  - templateName: Name of the template within the PDF file specified by fileSpec.
    Should be in PDFDocEncoding or Unicode.
  - bRename: If false, will add a /Rename key whose value is false, otherwise
    it won't (since true is the default). When renaming is indicated, all fields
	in the template modified by the FDF (for example, because new values for
	them are included) get renamed. The purpose of the /Rename key is to provide
	flexibility concerning whether fields get renamed as templates get spawned.
	The default is to do the renaming, in order to prevent potential conflicts
	between field names originating in different templates. The renaming scheme
	consists of prepending "P" + page # + ".template name_." + template # before
	the field name. For example, if a template called "flower" gets spawned into
	page 3, and this is the second template for that page in the FDF, then field
	"color" on that template gets renamed to "P3.flower_1.color".  However,
	the application designer can choose to include /Rename false if she knows
	that there are no name conflicts in the templates being used. Not doing the
	renaming simplifies her application, since she knows what the field names
	are, and can use them in cgi-scripts, JavaScripts, etc. without worrying
	about what they get renamed to.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcIncompatibleFDF,
	FDFErcEmbeddedFDFs, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFAddTemplate(FDFDoc theFDF,  ASBool bNewPage,
								const pdfFileSpec fileSpec,
								const char* templateName, ASBool bRename);


/*
  FDFExtractAttachment: Extracts the file uploaded by means of a "file selection" field,
  and creates a file out of it at the requested location.  Note: "extract" does not
  imply that the attachment gets removed from the FDF.
  Parameters:

  - theFDF
  - cFieldName
  - cFileName: Host-encoded string for the pathname of the file to create with
    the extracted attachment. If fileName is NULL, or an empty string, will save the file
	in the current folder, using the filename included with the attachment. However,
	if bIsFilePath is true and cFileName is not empty, will append the filename
	included with the attachment to cFileName and save the attachment there. For example,
	if you pass "c:\\temp\\" on a Windows system, and the filename found in the attachment
	is "photo.jpg", will save the file to c:\temp\photo.jpg.
  - bIsFilePath: See above
  - nFileName: If greater than zero, and cFileName is not NULL, FDFExtractAttachment will
    fill cFileName with the full pathname (up to nFileName bytes) where the attachment
	was saved (only if FDFExtractAttachment returns FDFErcOK).
  - cMimeType: Filled by FDFExtractAttachment(), if it returns FDFErcOK. Buffer
	containing the MIME type of the file. Can be null, if this information is of
	no interest to the caller.
  - nMimeTypeSize: The maximum number of bytes that can be written into cMimeType. If
	cMimeType is NULL, then bufSize is nMimeTypeSize.
  - Error codes: FDFErcBadParameter, FDFErcBadFDF, FDFErcIncompatibleFDF,
	FDFErcEmbeddedFDFs, FDFErcFieldNotFound, FDFErcNoValue, FDFErcFileSysErr,
	FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFExtractAttachment(FDFDoc theFDF, const char* cFieldName,
									  char* cFileName, ASInt32 nFileName,
									  ASBool bIsFilePath,
									  char* cMimeType, ASInt32 nMimeTypeSize);


/*
  FDFExtractAppendSaves: Extracts the incremental changes to the PDF that got submitted
  inside the FDF, and creates a file out of it at the requested location. Note:
  "extract" does not imply that the append-saves get removed from the FDF.
  Parameters:

  - theFDF
  - cFileName: Host-encoded string for the pathname of the file to create with
    the extracted bytes.
  - Error codes: FDFErcBadParameter, FDFErcIncompatibleFDF, FDFErcEmbeddedFDFs,
    FDFErcFileSysErr, FDFErcNoAppendSaves, FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFExtractAppendSaves(FDFDoc theFDF, const char* cFileName);


/*
  FDFAddDocJavaScript: Adds a script to the FDF, which will then get added to the doc-level
  scripts of a document, once the FDF gets imported into it. Parameters:

  - theFDF
  - cScriptName: Must either be in PDFDoc or in Unicode.
  - cScript: Must either be in PDFDoc or in Unicode. Hint: Use "\r" as line separator within
    the script. Use "\t" for tabs.
  - Error codes: FDFErcBadParameter, FDFErcIncompatibleFDF, FDFErcEmbeddedFDFs,
    FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFAddDocJavaScript(FDFDoc theFDF, const char* cScriptName, const char* cScript);


/*
  FDFSetOnImportJavaScript: Adds a script to the FDF, which will get executed when the FDF gets
  opened by Acrobat. You can set two such scripts: one that gets executed just before the data
  in the FDF is imported, and one that gets executed after. Hint: you would need to call
  FDFSetOnImportJavaScript twice to set both scripts. Parameters:

  - theFDF
  - cScript: Must either be in PDFDoc or in Unicode.
  - bBefore: Pass true to have cScript execute before the data in the FDF is imported. Pass false
    to indicate that it should be executed after the data gets imported.
  - Error codes: FDFErcBadParameter, FDFErcIncompatibleFDF, FDFErcEmbeddedFDFs,
    FDFErcInternalError
*/
FDFLIBAPI FDFErc FDFSetOnImportJavaScript(FDFDoc theFDF, const char* cScript, ASBool bBefore);



#ifdef __cplusplus
}
#endif

#endif /* _H_FDFTK */
